import os
import ffmpeg
from datetime import datetime
import tempfile
from scipy import stats

import tempfile

def generate_video(images, framerate, output_file, bitrate='20M'):
    ffmpeg \
        .input('concat:' + '|'.join([f'/www/wwwroot/ace/history/{image[1]}' for image in images])) \
        .output(output_file, r=framerate, vcodec='h264_nvenc',**{'b:v': bitrate}) \
        .run()


def main():
    tc_name = input("Enter the TC name: ")
    product_type = input("Enter the product type: ")
    bitrate = input("Enter the bitrate: ")

    directory = os.path.join(tc_name, product_type)
    images = []
    for root, dirs, files in os.walk(directory):
        for file in files:
            if file.endswith(".png"):
                try:
                    timestamp = datetime.strptime(file.split('_')[-1].split('.')[0], "%Y%m%d%H%M%S")
                    images.append((timestamp, os.path.join(root, file)))
                except ValueError:
                    print(f"Ignored {file} because its name does not match the expected format.")
    images.sort()

    total_frames = 0
    video_counter = 0
    # Define the possible intervals and their corresponding frame rates
    intervals = [(15, 4), (10, 6), (2.5, 24), (1, 60)]

    i = 0
    while i < len(images) - 1:
        clip_images = []
        time_diffs = []
        # Calculate the time differences for a group of images
        for j in range(i, min(i + 100, len(images) - 1)):
            time_diff = (images[j+1][0] - images[j][0]).total_seconds() / 60  # in minutes
            time_diffs.append(time_diff)
            clip_images.append(images[j])
        # Identify the interval that most time differences are closest to
        closest_interval = min(intervals, key=lambda x: abs(x[0] - stats.mode(time_diffs)[0]))
        framerate = closest_interval[1]
        output_file = f'output{video_counter}.mp4'
        print(f"Generating {output_file} with framerate {framerate}...")
        generate_video(clip_images, framerate, output_file, bitrate)
        print(f"Finished generating {output_file}.")
        total_frames += len(clip_images)
        i += len(clip_images)
        video_counter += 1

    print(f"Generated {video_counter} video clips.")

if __name__ == "__main__":
    main()
